/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-16 17:50:31.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.controller.v1;

import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.service.SmsService;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 认证中心相关接口
 *
 * @Version v1.0
 */
@RestController
@RequestMapping("/api/v1")
@Api(tags = "认证中心相关接口")
public class UserAuthRestController {
    @Autowired
    private UserAuthService userAuthService;
    @Autowired
    private SmsService smsService;

    @ApiOperation(value = "登录接口")
    @RequestMapping(value = "/user/login", method = {RequestMethod.GET, RequestMethod.POST})
    public Result login(@RequestParam String account, @RequestParam String password, @RequestParam Long rootOrgId) throws Exception {
        return userAuthService.login(account, password, rootOrgId);
    }

    @ApiOperation(value = "登出接口")
    @RequestMapping(value = "/user/logout", method = {RequestMethod.GET, RequestMethod.POST})
    public Result logout(@RequestHeader String key, @RequestHeader String token) throws Exception {
        return userAuthService.logout(key, token);
    }

    @ApiOperation(value = "获取用户信息接口")
    @RequestMapping(value = "/user/info", method = {RequestMethod.GET, RequestMethod.POST})
    public Result getUserInfo(@RequestHeader String key, @RequestHeader String token) throws Exception {
        return userAuthService.getUserInfo(key, token);
    }

    @ApiOperation(value = "修改密码接口")
    @RequestMapping(value = "/user/update/password", method = {RequestMethod.GET, RequestMethod.POST})
    public Result updatePassword(@RequestHeader String key, @RequestHeader String token, @RequestParam Long userId, @RequestParam String password) throws Exception {
        return userAuthService.updatePassword(key, token, userId, password);
    }

    @ApiOperation(value = "获取短信验证码接口")
    @RequestMapping(value = "/send/sms/code", method = {RequestMethod.GET, RequestMethod.POST})
    public Result sendSmsCode(@RequestHeader String key, @RequestHeader String token, @RequestParam String phone) throws Exception {
        return smsService.sendSmsCode(key, token, phone);
    }

    @ApiOperation(value = "校验短信验证码接口")
    @RequestMapping(value = "/check/sms/code", method = {RequestMethod.GET, RequestMethod.POST})
    public Result checkSmsCode(@RequestHeader String key, @RequestHeader String token, @RequestParam String phone, @RequestParam String code) throws Exception {
        return smsService.checkSmsCode(key, token, phone, code);
    }

}